﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

namespace Chaos
{
    public class GameState
    {
        Player player;
        List<Ground> groundObjects;
        List<Ground> groundObjectsDel;
        List<Ground> groundObjectsNew;
        List<Cloud> cloudObjects;
        List<Powerup> powerupObjects;
        List<Bush> bushObjects;
        public static List<Powerup> powerupObjectsDel;
        public static List<Bush> bushObjectsDel;
        Texture2D groundTexture;
        Texture2D cloud1, cloud2, cloud3, cloud4;
        Texture2D appleTexture;
        Texture2D playerTexture;
        Texture2D bushTexture;
        SpriteFont font;
        public static bool playerDead = false;
        public static int score = 0;
        GraphicsDevice gd;
        ContentManager cm;
        public static int coins = 0;
        SoundEffect purchaseSound;
        float rotationSpeed = 0.0f;

        public static bool doubleJump = false;
        public static int jumpAdd = 0;

        public GameState()
        {
            player = new Player();
            groundObjects = new List<Ground>();
            groundObjectsDel = new List<Ground>();
            groundObjectsNew = new List<Ground>();
            powerupObjects = new List<Powerup>();
            powerupObjectsDel = new List<Powerup>();
            bushObjects = new List<Bush>();
            bushObjectsDel = new List<Bush>();
            cloudObjects = new List<Cloud>();
        }

        public void LoadContent(GraphicsDevice graphicsDevice, ContentManager Content)
        {
            gd = graphicsDevice;
            cm = Content;
            groundTexture = Texture2D.FromFile(graphicsDevice, "Content/Grass.png");
            cloud1 = Texture2D.FromFile(graphicsDevice, "Content/Clouds/Cloud1.png");
            cloud2 = Texture2D.FromFile(graphicsDevice, "Content/Clouds/Cloud2.png");
            cloud3 = Texture2D.FromFile(graphicsDevice, "Content/Clouds/Cloud3.png");
            cloud4 = Texture2D.FromFile(graphicsDevice, "Content/Clouds/Cloud4.png");
            bushTexture = Texture2D.FromFile(graphicsDevice, "Content/Bush.png");
            playerTexture = Texture2D.FromFile(graphicsDevice, "Content/TwilightSprite.png");
            appleTexture = Texture2D.FromFile(graphicsDevice, "Content/Apple.png");
            purchaseSound = Content.Load<SoundEffect>("Sounds/Coin");
            font = Content.Load<SpriteFont>("Arial");
            Random r = new Random();
            for (int i = 0; i < 5; i++)
            {
                Ground g = new Ground(groundTexture, new Vector2(1000*i+(200*i), 300+(r.Next(100, 300))));
                g.width = 1000;
                groundObjects.Add(g);
            }
            for (int i = 0; i < 5; i++)
            {
                Cloud c = new Cloud(800+r.Next(0, 200), 10);
                c.Type = r.Next(1, 4);
                c.Speed = r.Next(2, 5);
                cloudObjects.Add(c);
            }
            player.LoadContent(graphicsDevice, Content);
        }

        public static void RemovePowerup(Powerup p)
        {
            powerupObjectsDel.Add(p);
        }

        public static void RemoveBush(Bush b)
        {
            bushObjectsDel.Add(b);
        }

        public void Update(GameTime gameTime)
        {
            if (!playerDead)
                score += player.speed / 4;
            KeyboardState ks = Keyboard.GetState();
            if (ks.IsKeyDown(Keys.Enter) && playerDead)
            {
                playerDead = false;
                player = new Player();
                groundObjects.Clear();
                bushObjects.Clear();
                powerupObjects.Clear();
                Random r = new Random();
                for (int i = 0; i < 5; i++)
                {
                    Ground g = new Ground(groundTexture, new Vector2(1000 * i + (200 * i), 300 + (r.Next(100, 300))));
                    g.width = 1000;
                    groundObjects.Add(g);
                }
                player.LoadContent(gd, cm);
                score = 0;
            }
            if (playerDead)
            {
                if (coins >= 10 && ks.IsKeyDown(Keys.D1))
                {
                    coins -= 10;
                    doubleJump = true;
                    purchaseSound.Play();
                }
                if (coins >= Math.Max(10, (jumpAdd / 30) * 10) && ks.IsKeyDown(Keys.D2))
                {
                    coins -= Math.Max(10, (jumpAdd / 30) * 10);
                    jumpAdd += 30;
                    purchaseSound.Play();
                }
            }
            rotationSpeed -= 0.05f;
            foreach (Ground g in groundObjects)
            {
                g.position.X -= player.speed;
                if (g.position.X < -g.width)
                {
                    groundObjectsDel.Add(g);
                    int i = groundObjects.Count;
                    Random r = new Random();
                    Ground newG = new Ground(groundTexture, new Vector2(1000 * i + 200, 300 + (r.Next(100, 300))));
                    newG.width = 1000;
                    if (r.NextDouble() < 0.1)
                    {
                        Powerup p = new Powerup();
                        p.texture = appleTexture;
                        p.X = 1000 * i + 700;
                        p.Y = (int)newG.position.Y-p.texture.Height;
                        powerupObjects.Add(p);
                    }
                    int numBushes = r.Next(0, 3);
                    if (numBushes > 0)
                    {
                        int offset = (newG.width - 100) / numBushes;
                        for (int j = 0; j < numBushes; j++)
                        {
                            Bush b = new Bush();
                            b.X = 1000 * i + 400 + (offset * j);
                            b.texture = bushTexture;
                            b.Y = (int)newG.position.Y - b.texture.Height;
                            bushObjects.Add(b);
                        }
                    }
                    groundObjectsNew.Add(newG);
                }
            }
            foreach (Powerup p in powerupObjects)
            {
                p.X -= player.speed;
                if (p.X < -100) powerupObjectsDel.Add(p);
            }
            foreach (Bush b in bushObjects)
            {
                b.X -= player.speed;
                if (b.X < -b.texture.Width) bushObjectsDel.Add(b);
            }
            foreach (Ground g in groundObjectsDel) groundObjects.Remove(g);
            foreach (Ground g in groundObjectsNew) groundObjects.Add(g);
            foreach (Powerup p in powerupObjectsDel) powerupObjects.Remove(p);
            foreach (Bush b in bushObjectsDel) bushObjects.Remove(b);
            foreach (Cloud c in cloudObjects)
            {
                c.Update();
                if (c.X < -300)
                {
                    c.Type = new Random().Next(1, 4);
                    c.Y = 10;
                    c.X = 1000;
                    c.Speed = new Random().Next(5, 10);
                }
            }
            groundObjectsDel.Clear();
            groundObjectsNew.Clear();
            if(!playerDead) player.Update(gameTime, groundObjects, powerupObjects, bushObjects);
        }

        public void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Begin(SpriteSortMode.Deferred, null, SamplerState.LinearWrap, null, null);
            foreach (Ground ground in groundObjects)
                ground.Draw(spriteBatch);
            foreach (Powerup p in powerupObjects)
                spriteBatch.Draw(p.texture, p.rect, Color.White);
            foreach (Bush b in bushObjects)
                spriteBatch.Draw(b.texture, b.rect, Color.White);
            foreach (Cloud c in cloudObjects)
            {
                switch (c.Type)
                {
                    case 1:
                        spriteBatch.Draw(cloud1, new Rectangle(c.X, c.Y, cloud1.Width, cloud1.Height), Color.White);
                        break;
                    case 2:
                        spriteBatch.Draw(cloud3, new Rectangle(c.X, c.Y, cloud2.Width, cloud2.Height), Color.White);
                        break;
                    case 3:
                        spriteBatch.Draw(cloud3, new Rectangle(c.X, c.Y, cloud3.Width, cloud3.Height), Color.White);
                        break;
                    case 4:
                        spriteBatch.Draw(cloud4, new Rectangle(c.X, c.Y, cloud4.Width, cloud4.Height), Color.White);
                        break;
                }
            }
            if (!playerDead)
                player.Draw(gameTime, spriteBatch);
            if (playerDead)
            {
                C3.XNA.Primitives2D.FillRectangle(spriteBatch, new Rectangle(0, 0, 800, 600), new Color(Color.Red, 0.1f));
                spriteBatch.DrawString(font, "You died!", new Vector2(10, 10), Color.White);
                spriteBatch.DrawString(font, 
                    "You ran " + score + " meters and earned " + Math.Floor((double)score/750) + " coins",
                    new Vector2(10,
                        10 + font.MeasureString("Score").Y),
                    Color.White);
                spriteBatch.Draw(
                    playerTexture, 
                    new Rectangle(400, 300, 125, playerTexture.Height),
                    new Rectangle(0, 0, 125, playerTexture.Height),
                    Color.White,
                    rotationSpeed,
                    new Vector2(87,playerTexture.Height/2),
                    SpriteEffects.None,
                    0);
                spriteBatch.DrawString(font, "Press enter to restart!", new Vector2(10, 10 + font.MeasureString("Score").Y * 2), Color.White);
                spriteBatch.DrawString(font, "UPGRADES", new Vector2(11, 401), Color.Black);
                spriteBatch.DrawString(font, "UPGRADES", new Vector2(10, 400), Color.White);
                spriteBatch.DrawString(font, "10 coins - double jump", new Vector2(10, font.MeasureString("UPGRADES").Y + 400), Color.White);
                spriteBatch.DrawString(font, coins + " coins", new Vector2(40 + font.MeasureString("UPGRADES").X, 400), Color.White);
                spriteBatch.DrawString(font,Math.Max(10, (jumpAdd / 30) * 10) + " coins - jump height", new Vector2(10, font.MeasureString("UPGRADES").Y + 420), Color.White);
                if(doubleJump)
                    C3.XNA.Primitives2D.DrawLine(spriteBatch,
                        new Vector2(10, font.MeasureString("UPGRADES").Y + 415),
                        new Vector2(10 + font.MeasureString("10 coins - double jump").X, font.MeasureString("UPGRADES").Y + 415),
                        Color.Black,
                        3.0f);
                else
                    spriteBatch.DrawString(font, 
                        "Press 1 to buy", 
                        new Vector2(50 + font.MeasureString("10 coins - double jump").X, 
                        font.MeasureString("UPGRADES").Y + 400),
                        Color.Purple);
                spriteBatch.DrawString(font,
                    "Press 2 to buy",
                    new Vector2(50 + font.MeasureString(Math.Max(10, (jumpAdd / 30) * 10)+" coins - jump height").X,
                    font.MeasureString("UPGRADES").Y + 420),
                    Color.Purple);
            }
            else
            {
                spriteBatch.DrawString(font, score + "m", new Vector2(800 - font.MeasureString(score + "m").X+1, 6), Color.Black);
                spriteBatch.DrawString(font, score + "m", new Vector2(800 - font.MeasureString(score + "m").X, 5), Color.White);
            }
            spriteBatch.End();
        }
    }
    public class Ground
    {
        public Color groundBase = new Color(146, 146, 146);
        public Texture2D groundTexture;
        public Vector2 position;
        public int width;
        public Ground(Texture2D groundTexture, Vector2 groundPosition)
        {
            this.position = groundPosition;
            this.groundTexture = groundTexture;
        }
        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(groundTexture,
                       new Rectangle(
                           (int)position.X,
                           (int)position.Y,
                           width,
                           groundTexture.Height),
                       new Rectangle(0, 0, width, groundTexture.Height),
                       Color.White);
            C3.XNA.Primitives2D.FillRectangle(spriteBatch,
                new Rectangle(
                    (int)position.X,
                    (int)position.Y + groundTexture.Height,
                    width,
                    600 - (int)position.Y - groundTexture.Height),
                groundBase);
        }
    }
    public class Cloud
    {
        public int X, Y;
        public int Speed = 3;
        public int Type = 1;
        public Cloud(int X, int Y)
        {
            this.X = X;
            this.Y = Y;
        }
        public void Update()
        {
            X -= Speed;
        }
    }
    public class Powerup
    {
        public Texture2D texture;
        public int X, Y;
        public Rectangle rect
        {
            get { return new Rectangle(X, Y, texture.Width, texture.Height); }
        }
    }
    public class Bush
    {
        public Texture2D texture;
        public int X, Y;
        public Rectangle rect
        {
            get { return new Rectangle(X, Y, texture.Width, texture.Height); }
        }
    }
}
