﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Audio;

namespace Chaos
{
    public class MLPGame : Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D TitleGraphic;
        Texture2D PressEnterGraphic;
        Texture2D CloudsGraphic;
        Texture2D PauseOverlayGraphic;
        Texture2D PauseButtonGraphic;
        public int CloudsY = 0;
        public static States state = States.TitleScreen;
        public static bool Paused = false;
        GameState mainGame;
        SoundEffect selectSound;
        SoundEffect song;
        SoundEffectInstance songInstance;
        SoundEffectInstance selectInstance;
        bool playGame = false;
        bool escapeWasDown = false;

        public MLPGame()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferHeight = 600;
            graphics.PreferredBackBufferWidth = 800;
            Content.RootDirectory = "Content";
            Window.Title = "Chaos game w/ twilight or something";
            IsMouseVisible = true;
            mainGame = new GameState();
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            TitleGraphic = Texture2D.FromFile(GraphicsDevice, "Content/Title.png");
            PressEnterGraphic = Texture2D.FromFile(GraphicsDevice, "Content/PressEnter.png");
            CloudsGraphic = Texture2D.FromFile(GraphicsDevice, "Content/Clouds.png");
            PauseButtonGraphic = Texture2D.FromFile(GraphicsDevice, "Content/PauseButton.png");
            PauseOverlayGraphic = Texture2D.FromFile(GraphicsDevice, "Content/PauseOverlay.png");
            selectSound = Content.Load<SoundEffect>("Sounds/Death");
            song = Content.Load<SoundEffect>("Sounds/MLGJ");
            songInstance = song.CreateInstance();
            songInstance.IsLooped = true;
            songInstance.Volume = 0.8f;
            songInstance.Play();
            selectInstance = selectSound.CreateInstance();
            mainGame.LoadContent(GraphicsDevice, Content);
        }

        protected override void Update(GameTime gameTime)
        {
            KeyboardState ks = Keyboard.GetState();
            MouseState ms = Mouse.GetState();
            if (ks.IsKeyDown(Keys.Enter) && state == States.TitleScreen)
            {
                selectInstance.Play();
                state = States.Game;
            }
            if (state == States.TitleScreen)
            {
                CloudsY += 2;
                if (CloudsY > 600) CloudsY = 0;
            }
            else if (state == States.Game)
            {
                if (ks.IsKeyDown(Keys.Escape)) escapeWasDown = true;
                if (ks.IsKeyUp(Keys.Escape) && escapeWasDown)
                {
                    escapeWasDown = false;
                    Paused = !Paused;
                }
                if(!Paused)
                    mainGame.Update(gameTime);
            }
            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            if (state == States.TitleScreen)
            {
                GraphicsDevice.Clear(Color.SkyBlue);
                spriteBatch.Begin();
                spriteBatch.Draw(CloudsGraphic, new Rectangle(0, CloudsY, 800, 600), Color.White);
                spriteBatch.Draw(CloudsGraphic, new Rectangle(0, CloudsY - 600, 800, 600), Color.White);
                spriteBatch.Draw(TitleGraphic, new Rectangle(0, 0, 800, 600), Color.White);
                //spriteBatch.Draw(PressEnterGraphic, new Rectangle(450, 230, 173, 17), Color.White);
                spriteBatch.End();
            }
            else if (state == States.Game)
            {
                GraphicsDevice.Clear(Color.LightGray);
                mainGame.Draw(gameTime, spriteBatch);
            }
            if (Paused)
            {
                spriteBatch.Draw(PauseOverlayGraphic, new Rectangle(0, 0, 800, 600), Color.White);
                spriteBatch.Draw(PauseButtonGraphic, new Rectangle(10, 10, 100, 120), Color.White);
            }
            base.Draw(gameTime);
        }
    }
    public enum States
    {
        TitleScreen,
        Game,
        GameOver
    }
}
