﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;

namespace Chaos
{
    public class Player
    {
        Texture2D spriteSheet;
        Texture2D speedMeter;
        int spriteSize = 130;
        int frame = 0;
        int frames = 0;
        int fpsCounter = 0;
        int X = 0, Y = 0;
        bool loop = true;
        bool moving = true;
        float gravity = 0.3f;
        public int speed = 6;
        PlayerState state = PlayerState.Walking;
        Vector2 startingPosition;
        Vector2 jumpSpeed;
        Vector2 jumpDirection;
        float downwardVelocity = 0.0f;
        float upwardVelocity = 6.5f;
        float forwardsVelocity = 0.1f;
        bool touchingGround = false;
        bool falling = false;
        SoundEffect deathSound;
        SoundEffect powerupSound;
        SoundEffect jumpSound;
        SoundEffect hitSound;
        List<Ground> groundObjects;
        SpriteFont font;
        bool displayPowerupMessage = true;
        bool doubleJumped = false;
        string powerupMessage = "";
        TimeSpan powerupMessageTime;

        public Player()
        {
            jumpSpeed = new Vector2(speed, speed);
            jumpDirection = Vector2.Zero;
        }

        public void LoadContent(GraphicsDevice graphicsDevice, ContentManager Content)
        {
            spriteSheet = Texture2D.FromFile(graphicsDevice, "Content/TwilightSprite.png");
            speedMeter = Texture2D.FromFile(graphicsDevice, "Content/SpeedMeter.png");
            deathSound = Content.Load<SoundEffect>("Sounds/Death");
            powerupSound = Content.Load<SoundEffect>("Sounds/Coin");
            jumpSound = Content.Load<SoundEffect>("Sounds/Jump");
            hitSound = Content.Load<SoundEffect>("Sounds/Hit");
            font = Content.Load<SpriteFont>("Arial");
            frames = (int)Math.Floor((double)spriteSheet.Width / spriteSize);
        }

        public void Update(GameTime gameTime, List<Ground> groundObjects, List<Powerup> powerupObjects, List<Bush> bushObjects)
        {
            if (moving || !loop)
            {
                fpsCounter += 1;
                if (fpsCounter > 2)
                {
                    if(loop || frame != 0)
                        frame += 1;
                    if (frame >= frames) frame = 0;
                    fpsCounter = 0;
                }
            }
            KeyboardState ks = Keyboard.GetState();
            foreach (Keys k in ks.GetPressedKeys())
                switch (k)
                {
                    case Keys.Space:
                        if (state == PlayerState.Jumping || falling) break;
                        state = PlayerState.Jumping;
                        startingPosition = new Vector2(X, Y);
                        jumpDirection.Y = jumpSpeed.Y;
                        jumpSound.Play();
                        moving = loop = false;
                        break;
                    default:
                        moving = loop = false;
                        break;
                }
            if(X < (800/2)-(spriteSize/2))
                X += speed;
            touchingGround = false;
            forwardsVelocity += 0.005f;
            if ((int)Math.Round(forwardsVelocity) >= 1 && speed <= 40)
            {
                speed += 1;
                forwardsVelocity = 0;
            }
            this.groundObjects = groundObjects;
            foreach (Ground g in groundObjects)
            {
                if(new Rectangle(
                    (int)g.position.X, 
                    (int)g.position.Y, 
                    g.width,
                    10).Intersects(new Rectangle(this.X+40, this.Y+spriteSheet.Height-10, spriteSize-40, 10))
                    /*|| this.X > g.position.X && this.X < g.position.X+g.width && this.Y >= g.position.Y && this.Y < g.position.Y + 5*/)
                {
                    touchingGround = true;
                    moving = loop = true;
                    falling = false;
                    break;
                }
            }
            foreach (Powerup p in powerupObjects)
            {
                if (p.rect.Intersects(new Rectangle(this.X + 40, this.Y, spriteSize - 40, spriteSheet.Height)))
                {
                    powerupSound.Play();
                    GameState.RemovePowerup(p);
                    powerupMessageTime = gameTime.ElapsedGameTime;
                    switch (new Random().Next(0, 10))
                    {
                        case 0:
                        case 1:
                        case 2:
                        case 3:
                            powerupMessage = "SPEED UP!";
                            displayPowerupMessage = true;
                            speed += new Random().Next(1, 3);
                            break;
                        case 4:
                        case 5:
                        case 6:
                            powerupMessage = "SLOW DOWN!";
                            displayPowerupMessage = true;
                            speed -= new Random().Next(1, 3);
                            break;
                        case 7:
                        case 8:
                        case 9:
                        case 10:
                            powerupMessage = "SCORE BOOST!";
                            displayPowerupMessage = true;
                            GameState.score += 1000;
                            break;
                    }
                }
            }
            foreach (Bush b in bushObjects)
            {
                if (b.rect.Intersects(new Rectangle(this.X + 40, this.Y, spriteSize - 40, spriteSheet.Height)))
                {
                    GameState.RemoveBush(b);
                    hitSound.Play();
                    if(speed > 3) speed -= 2;
                }
            }
            if (!touchingGround && state != PlayerState.Jumping)
            {
                downwardVelocity += gravity;
                Y += (int)downwardVelocity;
                moving = loop = false;
                falling = true;
            }
            else if(touchingGround)
            {
                downwardVelocity = 0.0f;
                moving = loop = true;
                falling = false;
            }
            if (state == PlayerState.Jumping)
            {
                if (startingPosition.Y - Y > 150 + GameState.jumpAdd)
                {
                    jumpDirection = Vector2.Zero;
                    state = PlayerState.Walking;
                    upwardVelocity = 3.0f;
                    downwardVelocity = 0.0f;
                    if (!ks.IsKeyDown(Keys.Space) || !GameState.doubleJump || doubleJumped)
                    {
                        falling = true;
                        doubleJumped = false;
                    }
                    else
                        doubleJumped = true;
                }
                else
                {
                    Y -= (int)jumpSpeed.Y + (int)upwardVelocity;
                    if(upwardVelocity > gravity) upwardVelocity -= gravity;
                }
            }
            if (displayPowerupMessage && gameTime.ElapsedGameTime.TotalSeconds - powerupMessageTime.TotalSeconds > 5)
                displayPowerupMessage = false;
            if (Y > 600)
            {
                GameState.playerDead = true;
                GameState.coins += (int)Math.Floor((double)GameState.score / 750);
                deathSound.Play();
            }
        }

        public void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(spriteSheet,
                new Rectangle(X, Y, spriteSize, spriteSheet.Height),
                new Rectangle(frame*spriteSize, 0, spriteSize, spriteSheet.Height), 
                Color.White);
            spriteBatch.Draw(speedMeter, new Rectangle(5, 5, (int)Math.Round(200 * ((double)speed / (double)40)), speedMeter.Height), Color.White);
            if (displayPowerupMessage)
                spriteBatch.DrawString(font, powerupMessage, new Vector2(5, 600 - font.MeasureString(powerupMessage).Y), Color.Red);
        }
    }
    public enum PlayerState
    {
        None,
        Walking,
        Jumping
    }
}
